import MIMEParser from "@vereign/mime-parser";
import { ICryptoService } from "../CryptoService/ICryptoService";
import { AttachmentData, AttachmentsSignatures, PartsRabinFingerprints, PartsSignatures, SignComponentsResult, SignedAttachmentsData, SigningServiceHelpers, VerifyComponentsResult } from "./types";
declare class SigningService {
    cryptoService: ICryptoService;
    helpers: SigningServiceHelpers;
    constructor();
    /**
     * Function implementing COMPONENTS signing routine
     * https://code.vereign.com/light/documentation/-/blob/master/Validation.md#components-mode
     * @param vendor - email vendor (GMAIL | OUTLOOK | ROUNDCUBE | GENERIC_MIME)
     * @param mime - mime string
     */
    signMimeComponents(vendor: string, mime: string | MIMEParser): Promise<SignComponentsResult>;
    parseMime(mime: string | MIMEParser, ignoreQrCodes?: boolean): Promise<{
        html: string;
        plain: string;
        attachmentsData: AttachmentData[];
    }>;
    getPartsSignatures(partsBatch: string[][]): Promise<string[]>;
    /**
     * Function implementing COMPONENTS signing routine
     * https://code.vereign.com/light/documentation/-/blob/master/Validation.md#components-mode
     * @param publicKeyPEM
     * @param vendor - email vendor (GMAIL | OUTLOOK | ROUNDCUBE | GENERIC_MIME)
     * @param mime - MIME string
     * @param partsSignatures
     * @param partsRabinFingerprints
     */
    verifyMimeComponents(publicKeyPEM: string, vendor: string, mime: string | MIMEParser, partsSignatures: PartsSignatures, partsRabinFingerprints: PartsRabinFingerprints): Promise<VerifyComponentsResult>;
    verifyPartsSignatures(publicKeyPEM: string, partsBatch: {
        signature: string;
        parts: string[];
    }[]): Promise<boolean[]>;
    /**
     * Function implementing specification
     * https://code.vereign.com/light/documentation/-/blob/master/Validation.md#partssignaturealgorithm
     *
     * @param parts - array of strings representing parts
     */
    preparePartsForSigning(parts: Array<string>): Promise<string>;
    hashAttachments(attachments: AttachmentData[], useHashingAlgorithm: boolean): Promise<AttachmentData[]>;
    /**
     * @param attachments - an array of attachments data objects
     */
    signAttachments(attachments: AttachmentData[]): Promise<SignedAttachmentsData>;
    /**
     * @param publicKeyPEM
     * @param attachments - an array of attachments hashes
     * @param signatures
     */
    verifyAttachments(publicKeyPEM: string, attachments: AttachmentData[], signatures: AttachmentsSignatures): Promise<boolean[]>;
}
export default SigningService;
