"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.arrayBufferToHex = exports.decompressData = exports.compressData = exports.ensureUint8Array = exports.ensureArrayBuffer = exports.ensureBase64 = exports.base64ToArrayBuffer = exports.arrayBufferToBase64 = void 0;
const zlib_min_1 = require("zlibjs/bin/zlib.min");
const arrayBufferToBase64 = (buffer) => {
    return Buffer.from(buffer).toString("base64");
};
exports.arrayBufferToBase64 = arrayBufferToBase64;
const base64ToArrayBuffer = (base64) => {
    return Buffer.from(base64, "base64");
};
exports.base64ToArrayBuffer = base64ToArrayBuffer;
const ensureBase64 = (data) => {
    return typeof data === "string" ? data : exports.arrayBufferToBase64(data);
};
exports.ensureBase64 = ensureBase64;
const ensureArrayBuffer = (data) => {
    return typeof data === "string" ? exports.base64ToArrayBuffer(data) : data;
};
exports.ensureArrayBuffer = ensureArrayBuffer;
const ensureUint8Array = (data) => {
    return data instanceof Uint8Array
        ? data
        : typeof data === "string"
            ? new Uint8Array(exports.base64ToArrayBuffer(data))
            : data instanceof ArrayBuffer
                ? new Uint8Array(data)
                : data;
};
exports.ensureUint8Array = ensureUint8Array;
const compressData = (binary) => {
    const deflate = new zlib_min_1.Zlib.Deflate(exports.ensureUint8Array(binary));
    return deflate.compress();
};
exports.compressData = compressData;
const decompressData = (binary) => {
    const inflate = new zlib_min_1.Zlib.Inflate(exports.ensureUint8Array(binary));
    return inflate.decompress();
};
exports.decompressData = decompressData;
const arrayBufferToHex = (buffer) => {
    return [...new Uint8Array(buffer)]
        .map((b) => b.toString(16).padStart(2, "0"))
        .join("");
};
exports.arrayBufferToHex = arrayBufferToHex;
exports.default = {
    compressData: exports.compressData,
    decompressData: exports.decompressData,
    arrayBufferToHex: exports.arrayBufferToHex,
    arrayBufferToBase64: exports.arrayBufferToBase64,
    base64ToArrayBuffer: exports.base64ToArrayBuffer,
    ensureUint8Array: exports.ensureUint8Array,
    ensureArrayBuffer: exports.ensureArrayBuffer,
    ensureBase64: exports.ensureBase64,
};
