import { ICryptoService } from "../CryptoService/ICryptoService";

export type AttachmentHashAlgorithm =
  | "sha1"
  | "sha256"
  | "sha384"
  | "sha512"
  | "md5";

export type AttachmentDataType = "raw" | "hash";

export interface AttachmentData {
  filename: string;
  content: string;
  type: AttachmentDataType;
  hashingAlgorithm?: AttachmentHashAlgorithm;
  size?: number;
  url?: string;
  signature?: string;
  contentType?: string;
}

export type SignedAttachmentsData = {
  [key: string]: AttachmentData[];
};

export type AttachmentsSignatures = {
  [key: string]: {
    signature: string;
    hashingAlgorithm: AttachmentHashAlgorithm;
  }[];
};

export interface PartsSignatures {
  htmlPartsSignature: string;
  plainPartsSignature: string;
  pseudoPlainPartsSignature: string;
  attachmentsPartsSignature: string;
}

export interface PartsRabinFingerprints {
  htmlPartRabinFingerprint: string;
  plainPartRabinFingerprint: string;
  pseudoPlainPartRabinFingerprint: string;
  htmlPartSize: number;
  plainPartSize: number;
  pseudoPlainPartSize: number;
}

export interface SignComponentsResult {
  partsSignatures: PartsSignatures;
  rabinFingerprints: PartsRabinFingerprints;
}

export interface VerifyComponentsResult {
  htmlPartsSignatureValid: boolean;
  plainPartsSignatureValid: boolean;
  pseudoPlainPartsSignatureValid: boolean;
  attachmentsPartsSignatureValid: boolean;
  severityLevel: number;
  statusMessage: string;
}
export interface SigningServiceHelpers {
  /**
   * External function that calculates RSA signature.
   * Private key is not exposed.
   * @param value
   */
  signRSAExternal: (value: string) => Promise<string>;
  parseHtml: (htmlString: string) => HTMLDocument;
  arrayBufferToBase64: (bytes: ArrayBuffer) => string;
  cryptoService: ICryptoService;
}
