import { Zlib } from "zlibjs/bin/zlib.min";

export const arrayBufferToBase64 = (buffer: ArrayBuffer): string => {
  return Buffer.from(buffer).toString("base64");
};

export const base64ToArrayBuffer = (base64: string): ArrayBuffer => {
  return Buffer.from(base64, "base64");
};

export const ensureBase64 = (data: string | ArrayBuffer): string => {
  return typeof data === "string" ? data : arrayBufferToBase64(data);
};

export const ensureArrayBuffer = (data: string | ArrayBuffer): ArrayBuffer => {
  return typeof data === "string" ? base64ToArrayBuffer(data) : data;
};

export const ensureUint8Array = (
  data: string | ArrayBuffer | Uint8Array
): Uint8Array => {
  return data instanceof Uint8Array
    ? data
    : typeof data === "string"
    ? new Uint8Array(base64ToArrayBuffer(data))
    : data instanceof ArrayBuffer
    ? new Uint8Array(data)
    : data;
};

export const compressData = (binary: string | ArrayBuffer): ArrayBuffer => {
  const deflate = new Zlib.Deflate(ensureUint8Array(binary));
  return deflate.compress();
};

export const decompressData = (binary: string | ArrayBuffer): ArrayBuffer => {
  const inflate = new Zlib.Inflate(ensureUint8Array(binary));
  return inflate.decompress();
};

export const arrayBufferToHex = (buffer: ArrayBuffer): string => {
  return [...new Uint8Array(buffer)]
    .map((b) => b.toString(16).padStart(2, "0"))
    .join("");
};

export default {
  compressData,
  decompressData,
  arrayBufferToHex,
  arrayBufferToBase64,
  base64ToArrayBuffer,
  ensureUint8Array,
  ensureArrayBuffer,
  ensureBase64,
};
